function Alpha = func_topic_init(Xid,Xcnt,num_topics,voc,scale)
% DBHTM_INIT initializes the DBHTM algorithm using a tree-structured k-means algorithm.
%
% Written by Aonan Zhang(az2385@columbia.edu); John
% Paisley(jpaisley@columbia.edu)

D = length(Xid);
X = zeros(voc,D);
for d = 1:D
    X(Xid{d},d) = Xcnt{d}'/sum(Xcnt{d});
end

num_ite = 3;

[B,c] = K_means_L1(X,num_topics,num_ite);
%[B,c] = R_assign(X,num_topics);
Alpha = scale*B';

for i = 1:num_topics
    vec = gamrnd(ones(1,voc),1);
    if isnan(Alpha(i,:))>0
        Alpha(i,:) = scale*vec/sum(vec);
    else
        Alpha(i,:) =.95*Alpha(i,:) + .05*scale*vec/sum(vec);
    end
end

dic_size = 512;
Alpha_tag = Alpha(:,(dic_size+1):end);
tag_avg = sum(Alpha_tag,2)/(size(Alpha_tag,2));
Alpha(:,(dic_size+1):end) = repmat(tag_avg, 1, size(Alpha_tag,2));

% K-Means algorithm with L1 assignment and L2 mean minimization
function [B,c] = K_means_L1(X,K,maxite)

D = size(X,2);
[a,b] = sort(rand(1,D));
if D >= K
    B = X(:,b(1:K));
else
    B = rand(size(X,1),K);
    B = B./repmat(sum(B,1),size(B,1),1);
end
c = zeros(1,D);

for ite = 1:maxite
    for d = 1:D
        [a,c(d)] = min(sum(abs(B - repmat(X(:,d),1,K)),1));
    end
    for k = 1:K
        B(:,k) = mean(X(:,c==k),2);
    end
end

cnt = histc(c,1:K);
[t1,t2] = sort(cnt,'descend');
B = B(:,t2);
c2 = zeros(1,length(c));
for i = 1:length(t2)
    idx = find(c == t2(i));
    c2(idx) = i;
end
c = c2;

function [B,c] = R_assign(X,K)

D = size(X,2);
c = unidrnd(K,1,D);
for k = 1:K
    B(:,k) = mean(X(:,c==k),2);
end

cnt = histc(c,1:K);
[t1,t2] = sort(cnt,'descend');
B = B(:,t2);
c2 = zeros(1,length(c));
for i = 1:length(t2)
    idx = find(c == t2(i));
    c2(idx) = i;
end
c = c2;

