function [model g_bound] = markov_mm( doc_id, doc_cnt, hyper, o_model, rho, g_ite, Alpha_init )
    
    if isempty(o_model)
        ntopic = hyper.topic; 
        nvocab = hyper.vocab;
        ndoc = length(doc_id);
        Alpha = Alpha_init;
        A = hyper.alpha*ones(ntopic,ntopic)...
            +func_drchrnd(10*ones(1,ntopic),ntopic);
        A_0 = hyper.alpha*ones(1,ntopic)+func_drchrnd(10*ones(1,ntopic),1);
        tau = hyper.tau;
        t_length = hyper.t_length;
    else
        ntopic = o_model.ntopic;
        nvocab = o_model.nvocab;
        ndoc = length(doc_id);
        Alpha = o_model.Alpha;
        A = o_model.A;
        A_0 = o_model.A_0;
        tau = o_model.tau;
        t_length = o_model.t_length;
    end
    
    % global iterations
    stats_cond_z0a=zeros(1,ntopic);
    stats_cond_za=zeros(ntopic,ntopic);
    Eln_td = psi(sum(Alpha,2));
    % for each document d
    Alpha_upd = zeros(ntopic,nvocab);
    topic_porp = zeros(ntopic,ndoc);
    exp_pair_temp0 = (exp(psi(A_0)-psi(sum(A_0))))';
    exp_pair_temp = exp(psi(A)-repmat(psi(sum(A,2)),1,ntopic));
    g_bound = 0;tic;
    for d=1:ndoc
        lambda_d = ones(t_length-1,1);
        tau_d = tau*ones(t_length-1,1);
        N_d = length(doc_id{d});
        ids = doc_id{d};
        cnts = doc_cnt{d};
        Alpha_sub = Alpha(:,ids);
        Eln_w = psi(Alpha_sub) - repmat(Eln_td,1,length(ids));
        
        prob_l = func_init(Eln_w,cnts,ntopic,t_length);

        o_bound = 1; conv = 0; i_ite = 1;
        while ~conv
            % update potentials
            single_pot=[];pair_pot=[];
            
            log_pot = sum(repmat(prob_l(1,:).*cnts,ntopic,1).*Eln_w,2);
            single_pot{1} = exp(log_pot-max(log_pot));
            pair_pot{1} = exp_pair_temp0;
            
            for i=2:t_length
                log_pot = sum(repmat(prob_l(i,:).*cnts,ntopic,1).*Eln_w,2);
                single_pot{i} = exp(log_pot-max(log_pot));
                pair_pot{i} = exp_pair_temp;
            end
           
            [ s_marg, p_marg, sum_cond ] = func_forward_backward( single_pot, pair_pot );

            % for each word n in document d
            % update ell
            Eln_u = psi(lambda_d)-psi(lambda_d+tau_d);
            Eln_1_u = psi(tau_d)-psi(lambda_d+tau_d);
            Eln_nu = Eln_u + cumsum(Eln_1_u)-Eln_1_u;
            Eln_nu = [Eln_nu; sum(Eln_1_u)];

            prob_l = repmat(Eln_nu,1,N_d)+s_marg'*Eln_w;
            prob_l = exp(prob_l);
            prob_l = prob_l./repmat(sum(prob_l,1),t_length,1);
            
            % update u
            sum_prob_l = sum(prob_l.*repmat(cnts,t_length,1),2);%prob_1*cnts';
            lambda_d = ones(t_length-1,1)+sum_prob_l(1:(end-1));
            l_sum = cumsum(sum_prob_l(end:-1:1));
            l_sum = l_sum(end:-1:1)-sum_prob_l;
            tau_d = tau*ones(t_length-1,1)+l_sum(1:(end-1));
            % compute the variational bound
            %bound = func_doc_bound( s_marg, p_marg, A_0, A, tau, Eln_u, Eln_1_u,...
            %   lambda_d, tau_d, prob_l, Eln_w, Eln_nu, cnts );
            %if isnan(bound)
            %    disp(d);
            %end
            %disp(num2str(bound));
            if i_ite>10%((abs((bound-o_bound)/o_bound)<10^-3.5) && i_ite>10) || i_ite>20
                conv = 1;
                temp_nu = exp(Eln_nu)/(sum(exp(Eln_nu)));
                topic_porp(:,d) = s_marg*temp_nu;
                stats_cond_za = stats_cond_za + sum_cond;
                stats_cond_z0a = stats_cond_z0a + s_marg(:,1)';
                Alpha_upd(:,ids) = Alpha_upd(:,ids) + s_marg*(prob_l.*repmat(cnts,t_length,1));
                if rem(g_ite, 10) == 0
                    bound = func_doc_bound( s_marg, p_marg, A_0, A, tau, Eln_u, Eln_1_u,...
                        lambda_d, tau_d, prob_l, Eln_w, Eln_nu, cnts );
                    g_bound = g_bound + bound;
                end
            end
            %o_bound = bound;
            i_ite = i_ite+1;
        end
    end
    
    % compute total bound
    % g_bound = g_bound + func_global_bound(A_0, A, Alpha, hyper);
    if rem(g_ite, 10) == 0
        disp(['Iter:' num2str(g_ite) '  Bound:' num2str(g_bound) '  Time:' num2str(toc)  ]);
    else
        disp(['Iter:' num2str(g_ite) '  Time:' num2str(toc)  ]);
    end
    
    % update global parameters
    upt_A_0 = hyper.alpha*ones(1,ntopic) + 100000/ndoc*stats_cond_z0a;
    upt_A = hyper.alpha*ones(ntopic,ntopic) + 100000/ndoc*stats_cond_za;
    upt_Alpha = hyper.alpha0*ones(ntopic,nvocab) + 50000*Alpha_upd;
    A_0 = (1-rho)*A_0 + rho*upt_A_0;
    A = (1-rho)*A + rho*upt_A;
    Alpha = (1-rho)*Alpha + rho*upt_Alpha;
            
    model=[];
    model.ntopic = ntopic;
    model.nvocab = nvocab;
    model.Alpha = Alpha;
    model.A = A;
    model.A_0 = A_0;
    model.tau = tau;
    model.t_length = t_length;
end

